/**
 * @author 秃破天际
 * @version 10.0
 * Created by 秃破天际 on 2020-01-16
 **/
!(function () {
    var Model = BI.inherit(Fix.Model, {
        _init: function () {
            this.service = BI.Services.getService("dec.service.data.set");
        },
        context: ["dataSetName", "ableSave"],
        state: function () {
            var val = this.options.value.datasetData || {};
            return {
                loaderItems:[],
                resolverItems:[],
                cacheItems:[],
                cacheType: val.cacheType || 0,
                cacheTime: val.cacheTime || 0,
                loaderType : val.loaderType || "",
                loaderConfig : val.loaderConfig || "",
                resolverConfig : val.resolverConfig || "",
                resolverType : val.resolverType || "",
                parameters: val.parameters || []
            };
        },
        childContext: ["previewAble", "previewedDataSet"],
        watch: {
            previewAble: function (v) {
                this.setAbleSave(v);
            }
        },
        computed: {
            paramHeader: function () {
                //定义参数列表表头
                var map = BI.map(BI.Constants.getConstant("dec.constants.data.set.sql.header"), function (i, v) {
                    return BI.extend({
                        textAlign: "left",
                        height: 30,
                        hgap: 10
                    }, v);
                });
                map.push({
                    textAlign: "left",
                    height: 30,
                    hgap: 10,
                    text: ""
                });
                return [map];
            },
            paramItems: function () {
                var self = this;
                return BI.map(this.model.parameters, function (i, param) {
                    return [{
                        type: "bi.text_editor",
                        value: param.name,
                        textAlign: "left",
                        height: 30,
                        lgap: 10,
                        listeners: [{
                            eventName: BI.TextEditor.EVENT_CHANGE,
                            action: function () {
                                self.setParamName(i, this.getValue());
                            }
                        }]
                    }, {
                        type: "bi.icon_text_value_combo",
                        cls: "field-type-change",
                        height: 30,
                        items: BI.Constants.getConstant("dec.constants.data.set.sql.field"),
                        value: param.type,
                        listeners: [{
                            eventName: BI.IconTextValueCombo.EVENT_CHANGE,
                            action: function () {
                                self.setParamType(i, this.getValue()[0]);
                            }
                        }]
                    }, self.service.createParameterValueItem(param, function (val) {
                        self.setParamValue(i, val);
                    }),{
                        type: "bi.icon_button",
                        cls: "default-delete-font",
                        height: 30,
                        lgap: 10,
                        title: BI.i18nText("Dec-Basic_Delete"),
                        handler: function () {
                            self.deleteParameter(i);
                        }
                    }];
                });
            },
            previewAble: function () {
                return BI.isNotEmptyString(this.model.loaderType)
                    && BI.isNotEmptyString(this.model.loaderConfig)
                    && BI.isNotEmptyString(this.model.resolverType)
                    && BI.isNotEmptyString(this.model.resolverConfig);
            },
            previewedDataSet: function () {
                return {
                    datasetType: DecCst.DataSet.Type.FACTORY,
                    datasetName: this.model.dataSetName,
                    datasetData: {
                        loaderType: this.model.loaderType,
                        loaderConfig: this.model.loaderConfig,
                        resolverType: this.model.resolverType,
                        resolverConfig: this.model.resolverConfig,
                        cacheType:this.model.cacheType,
                        cacheTime:this.model.cacheTime,
                        parameters: this.model.parameters
                    }
                };
            }
        },
        _getConfig: function(items,type){
            for( var i=0,len=items.length; i<len; i++ ){
                if( items[i].value == type ){
                    return items[i].dftConfig;
                }
            }
            return "{}";
        },
        actions: {
            initPage: function (cb) {
                var self = this;
                Dec.Utils.getDataFactoryInitConfig(function (data) {
                    self.model.loaderItems = BI.map(data.data.loaderItems, function (i, v) {
                        v.text = BI.i18nText(v.text);
                        return v;
                    });
                    self.model.resolverItems = BI.map(data.data.resolverItems, function (i, v) {
                        v.text = BI.i18nText(v.text);
                        return v;
                    });
                    self.model.cacheItems = BI.map(data.data.cacheItems, function (i, v) {
                        v.text = BI.i18nText(v.text);
                        return v;
                    });
                    if (BI.isEmptyString(self.model.cacheType)) {
                        self.model.cacheType = BI.size(data.data.cacheItems) > 0 ? data.data.cacheItems[0].value : "";
                    }
                    if (BI.isEmptyString(self.model.loaderType)) {
                        self.model.loaderType = BI.size(data.data.loaderItems) > 0 ? data.data.loaderItems[0].value : "";
                        self.model.loaderConfig = self._getConfig(self.model.loaderItems,self.model.loaderType);
                    }
                    if (BI.isEmptyString(self.model.resolverType)) {
                        self.model.resolverType = BI.size(data.data.resolverItems) > 0 ? data.data.resolverItems[0].value : "";
                        self.model.resolverConfig = self._getConfig(self.model.resolverItems,self.model.resolverType);
                    }
                    cb();
                });
            },
            setCacheType: function(v){
                this.model.cacheType = v;
            },
            setCacheTime: function(v){
                this.model.cacheTime = v;
            },
            setLoaderType: function(v){
                this.model.loaderType = v;
                this.model.loaderConfig =this._getConfig(this.model.loaderItems,v);
            },
            setResolverType: function(v){
                this.model.resolverType = v;
                this.model.resolverConfig = this._getConfig(this.model.resolverItems,v);
            },
            setLoaderConfig: function(v){
                this.model.loaderConfig = v;
            },
            setResolverConfig: function(v){
                this.model.resolverConfig = v;
            },
            refreshParam: function () {
                var self = this;
                Dec.Utils.getDataSetParameters(this.model.previewedDataSet, function (res) {
                    self.model.parameters = self.service.getParameters(res.data, self.model.parameters);
                });
            },
            addParamter:function(v){
                this.model.parameters.push(v);
            },
            deleteParameter:function(index){
                this.model.parameters.splice(index, 1);
            },
            setParamName: function (index, name) {
                this.model.parameters[index].name = name;
            },
            setParamType: function (index, type) {
                if (this.model.parameters[index].type !== type) {
                    this.model.parameters[index].type = type;
                    this.model.parameters[index].value = this.service.getDefaultValueByType(type);
                    this.model.parameters.splice(0, 0);
                }
            },
            setParamValue: function (index, val) {
                this.model.parameters[index].value = val;
            },
            setAbleSave: function (v) {
                this.model.ableSave = v;
            }
        }
    });
    BI.model("dec.model.data.set.type.factory", Model);
})();